def new_url_with_parameters(url, params_dict):
    """
    Returns a new URL with an added query string, described by params_dict.
    @param params_dict: a dictionary with all the parameters to add
    @param path: url to add the parameters to
    @return: the url (a string) with given parameters 
    """
    #Derived from <http://djangosnippets.org/snippets/1627/>
    
    def param_bit(key, value):
        if value:
            return "%s=%s" % (key, value)
        else:
            return "%s" % key
    
    if len(params_dict):
        url_params = list()
        url += "?%s" % "&".join([param_bit(key, value) for (key, value) in params_dict.items()])
    
    return url

def current_url_with_parameters(request, new_params_dict):
    """
    Returns the current URL with some parameters changed, which are
    described in new_params_dict. The rest of the query string remains
    intact.
    """
    params = request.GET.copy() #this includes parameters that aren't used by the FilterSystem
    params.update(new_params_dict)
    url = request.path
    return new_url_with_parameters(url, params)

