from harvest.filters import filters, containers
from harvest.common.ordereddict import OrderedDict
import models

class PkgNameFilter(filters.EditFilter):
    def process_queryset(self, queryset):
        return queryset.filter(name__startswith = self.get_value())

class PkgSetFilter(filters.ChoiceFilter):
    def default_choices_dict(self):
        choices_dict = OrderedDict()
        for s in models.PackageSet.objects.all(): #TODO: find a way to sort these
            choices_dict[s.name] = s
        return choices_dict
    
    def process_queryset(self, queryset):
        return queryset.filter(packagesets__in = self.get_selected_choices())



class OppFeaturedFilter(filters.Filter):
    def process_queryset(self, queryset):
        return queryset.filter(opportunitylist__featured = True)

class OppListFilter(filters.ChoiceFilter):
    def default_choices_dict(self):
        choices_dict = OrderedDict()
        for l in models.OpportunityList.objects.all(): #TODO: find a way to sort these
            choices_dict[l.name] = l
        return choices_dict
    
    def process_queryset(self, queryset):
        return queryset.filter(opportunitylist__in = self.get_selected_choices())


#we don't really need to create a special type here, but it may be handy
class HarvestFilters(containers.FilterSystem):
    def __init__(self):
        super(HarvestFilters, self).__init__(
            [
                filters.FilterGroup('pkg', [
                    PkgNameFilter('name', name='Named'),
                    PkgSetFilter('set', name='Package sets')
                ], name='Packages' ),
                filters.FilterGroup('opp', [
                    OppFeaturedFilter('featured', name='Featured'),
                    OppListFilter('list', name='Selected')
                ], name='Opportunities' )
            ],
            default_parameters = { 'pkg' : 'set',
                                   'pkg:set' : 'ubuntu-desktop',
                                   'opp' : 'list',
                                   'opp:list' : 'bitesize' }
            #TODO: change to no defaults, detect that case in view and templates and provide helpful instructions in the results pane.
        )

